#' Setup GitHub Actions to check package
#' 
#' @description 
#' This function creates a configuration file (`.yaml`) to setup GitHub Actions 
#' to check the package. This file will be written (from a template adapted 
#' from [usethis::use_github_action()]) in the folder 
#' `.github/workflows/R-CMD-check.yaml`.
#' 
#' @param open a logical value
#' 
#'   If `TRUE` (default) the file is opened in the editor.
#' 
#' @param overwrite a logical value
#' 
#'   If this file is already present and `overwrite = TRUE`, it will be erased 
#'   and replaced. Default is `FALSE`.
#'   
#' @param quiet a logical value
#' 
#'   If `TRUE` messages are deleted. Default is `FALSE`.
#'   
#' @details 
#' This workflow runs `R CMD check` on the three major operating systems
#' (Ubuntu, macOS, and Windows) on the latest release of R. If user want to
#' change this setting he can replace this `.yaml` file be another (e.g. 
#' generated by running [usethis::use_github_action()]).
#' 
#' @return None
#'
#' @export
#' 
#' @family development functions
#'
#' @examples
#' \dontrun{
#' add_github_actions_check()
#' }

add_github_actions_check <- function(open = FALSE, overwrite = FALSE,
                                     quiet = FALSE) {

  
  stop_if_not_logical(open, overwrite, quiet)
  
  if (!dir.exists(file.path(path_proj(), ".git"))) {
    stop("The project is not versioning with git.")
  }
  
  
  path <- file.path(path_proj(), ".github", "workflows", "R-CMD-check.yaml")
  
  
  ## Do not replace current file but open it if required ----
  
  if (file.exists(path) && !overwrite) {
    
    if (!open) {
      
      stop("An '.github/workflows/R-CMD-check.yaml' file is already present. ",
           "If you want to replace it, please use `overwrite = TRUE`.")
      
    } else {
      
      edit_file(path)
      return(invisible(NULL))
    }
  }
  
  
  ## Copy Template ----
  
  dir.create(file.path(path_proj(), ".github", "workflows"), 
             showWarnings = FALSE, recursive = TRUE)

  invisible(
    file.copy(system.file(file.path("templates", "__RCMDCHECK__"), 
                          package = "rcompendium"), path))

  
  if (!quiet) 
    ui_done("Writing {ui_value('.github/workflows/R-CMD-check.yaml')} file")
  
  if (open) edit_file(path)
  
  invisible(NULL)
}
